@@ -1,27 +1,35 @@
 /*
-  Xinvest Copyright (c)1995-98, 2000 
-  Mark Buser, All Rights Reserved.
-
-  Permission is hereby granted to copy
-  and freely distribute copies of this
-  program for non-commercial purposes
-  without fee, provided that this notice
-  appears in all copies.
-  
-  All redistributions must be in their
-  entirety as originally distributed.
-  Feel free to modify Xinvest, but
-  modified versions may not be distributed
-  without prior consent of the author.
-  
-  This software is provided 'as-is'
-  without any express or implied warranty.
-  In no event will the author be held
-  liable for any damages resulting from
-  the use of this software.
-
-  $Revision: 2.32 $ $Date: 2000/02/04 22:47:26 $
-*/
+ * Copyright (c) 1996, 1997, 1998, 2000, 2001, Mark Buser.
+ * Copyright (c) 2001, 2002, 2003, 2004, Danny Backx.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are
+ * met:
+ *
+ * Redistributions of source code must retain the above copyright notice,
+ * this list of conditions and the following disclaimer.
+ * Redistributions in binary form must reproduce the above copyright
+ * notice, this list of conditions and the following disclaimer in the
+ * documentation and/or other materials provided with the distribution.
+ * Neither the names the authors (see above), nor the names of other
+ * contributors may be used to endorse or promote products derived from
+ * this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * $Header: /pack/anoncvs/xinvest/src/account.c,v 2.42 2004/04/29 19:14:37 danny Exp $
+ */
 
 #include <stdlib.h>
 #include <string.h>
@@ -42,7 +50,9 @@
 #include "transP.h"
 #include "trans.h"
 #include "util.h"
+#include "xutil.h"
 
+#include "xinvest.h"
 
 typedef struct accnt {
 
@@ -52,7 +62,7 @@ typedef struct accnt {
   Widget textWidget;      /* widget holding transaction listing */
   char *transText;        /* if the text is not display, its here */
   char *filename;         /* where from */
-  unsigned flags;         /* modified, applied, in portfolio, writable */
+  unsigned flags;         /* modified, applied, in portfolio, writable, live-update */
   NAV  *calc_nav;         /* current date, price */
   char *title;            /* title */
   char *ticker;           /* ticker symbol */
@@ -63,6 +73,8 @@ typedef struct accnt {
   int  transno;           /* # transactions */
   TRANS *trans;           /* the transactions */
   PLOT_VAR  *plot;        /* the plot variables selected and attributes */
+
+  int			currency_account;
 } ACCOUNT;
 
 static int accountno = 0;
@@ -74,10 +86,6 @@ static void accountDirty();
 static void makeAccountMenu();
 static void setAccountMenuItem ( int );
 
-extern Widget Toplevel;   /* Dialog parent  */
-extern Widget Transtext;  /* Transaction text widget */
-
-
 /* Where the apply change button is on the account menu */
 #define APPLY_BUTTON "button_2"
 
@@ -169,7 +177,7 @@ int newAccount ( char *filename, char *transtext, int writable )
   /* Create new account */
   /* +2 == one for [0] (unused), one for new account */
   account = (ACCOUNT *) XtRealloc ( (char *)account, 
-                                    sizeof(ACCOUNT) * (accountno+2) ); 
+                                    sizeof(ACCOUNT) * (accountno+60)); 
   if (account == NULL) {
     write_status ("Out of memory: could not create new account.", ERR);
     return(0);
@@ -216,6 +224,7 @@ int newAccount ( char *filename, char *transtext, int writable )
   account[accountno].transno = 0;
   account[accountno].trans = NULL;
   account[accountno].plot = makeGraphVar();
+  account[accountno].currency_account = 0;
 
   return(accountno);
 }
@@ -228,7 +237,7 @@ void closeAccount ( int index )
     /* Has account been modified */
     if (account[index].flags & ACCOUNT_DIRTY ) {
       char prompt[] = "Account has been changed, close anyway?";
-      if ( AskUser ( Toplevel, prompt, "OK", 
+      if ( AskUser ( per->Toplevel, prompt, "OK", 
                      "Cancel", YES) == NO )
         return;
     }
@@ -237,7 +246,7 @@ void closeAccount ( int index )
     if ( account[index].filename && 
          account[index].flags & ACCOUNT_IN_PORTFOLIO ) {
        char prompt[] = "Account is in the portfolio, close anyway?";
-       if ( AskUser ( Toplevel, prompt, "OK", "Cancel", YES) == NO )
+       if ( AskUser ( per->Toplevel, prompt, "OK", "Cancel", YES) == NO )
          return;
        else {
          portAccount( index, PORT_DELETE);
@@ -286,19 +295,20 @@ void closeAccount ( int index )
       account[index].transno    = account[index+1].transno;
       account[index].trans      = account[index+1].trans;
       account[index].plot       = account[index+1].plot;
+      account[index].currency_account = account[index+1].currency_account;
       index++;
     }
 
     accountno--;
     account = (ACCOUNT *)XtRealloc ( (char *)account, 
-                                     sizeof(ACCOUNT) * (accountno+1) ); 
+                                     sizeof(ACCOUNT) * (accountno+60) ); 
 
     /* The above should never fail, but boy if it does! */
     if (account == NULL) {
       char prompt[] = "Memory error removing account. Account\n"
                       "pointer is corrupt. Exiting Xinvest without\n"
                       "saving changes.\n";
-      AskUser ( Toplevel, prompt, "OK", "Cancel", YES );
+      AskUser ( per->Toplevel, prompt, "OK", "Cancel", YES );
       exit(-1);
     }
 
@@ -319,10 +329,14 @@ void processAccount ( int num )
   if (num == 0)
     return;
 
+#if 0
+  fprintf(stderr, "ProcessAccount(%d,%s)\n", num, account[num].filename);
+#endif
+
   /* Is it in the porfolio? Ask user if we should refresh */
   if (account[num].flags & ACCOUNT_IN_PORTFOLIO ) {
     char prompt[] = "Account is in the portfolio, update?";
-    if ( AskUser ( Toplevel, prompt, "OK", "Cancel", YES) == NO )
+    if ( AskUser ( per->Toplevel, prompt, "OK", "Cancel", YES) == NO )
       return;
     else {
       portAccount( num, PORT_DELETE);
@@ -346,7 +360,7 @@ void processAccount ( int num )
     if ((back = strchr(transtext, '\n')))
       *back = '\0';                 /* write NULL to end of 1st line */
     /* Always process once, even if its NULL, to clear out trans struct */
-    error = rd_trans (front, T_NEW);
+    error = rd_trans(num, front, T_NEW);
 
     /* 2nd+ line, look for newlines to feed to rd_trans */
     if (back && (front = back+1) < (transtext+len)) {
@@ -356,7 +370,7 @@ void processAccount ( int num )
     }
     while (back && !error) {
       *back = '\0';                   /* write NULL to end of line */
-      error = rd_trans (front, T_SAME);
+      error = rd_trans(num, front, T_SAME);
       if ((front = back+1) < (transtext+len)) {
         back = strchr (front, '\n');
       } else {
@@ -402,6 +416,25 @@ void processAccount ( int num )
   /* We're up to date */
   account[num].flags &= ~ACCOUNT_UPDATE;
 
+  if (account[num].cticker) {
+	  int	i;
+	  /* Figure out where our currency ticker is */
+#if 0
+	  fprintf(stderr, "ProcessAccount(%s) currency [%s]\n",
+			  account[num].title, account[num].cticker);
+#endif
+	  account[num].currency_account = 0;
+	  for (i=1; i<=accountno; i++)
+		  if (account[i].ticker &&
+				  strcmp(account[i].ticker, account[num].cticker) == 0) {
+			  account[num].currency_account = i;
+#if 0
+			  fprintf(stderr, "\t--> found account %d\n", i);
+#endif
+			  break;
+		  }
+  }
+
   /* Fill in structure if we have transactions */
   if ( account[num].transno ) {
 
@@ -440,6 +473,29 @@ void processAccount ( int num )
 
 }
 
+void ProcessAccountsForCticker(void)
+{
+	  int	i, num;
+
+	  for (num = 1; num <= accountno; num++)
+	  	if (account[num].cticker && !account[num].currency_account) {
+			/* Figure out where our currency ticker is */
+#if 0
+			fprintf(stderr, "ProcessAccountsForCticker(%s) currency [%s]\n",
+				account[num].title, account[num].cticker);
+#endif
+			account[num].currency_account = 0;
+			for (i=1; i<=accountno; i++)
+				if (strcmp(account[i].ticker, account[num].cticker) == 0) {
+					account[num].currency_account = i;
+#if 0
+					fprintf(stderr, "\t--> found account %d\n", i);
+#endif
+					break;
+				}
+		}
+}
+
 void displayAccount (int num)
 {
   /* All the stupid widgets we need to set */
@@ -827,7 +883,7 @@ void accountCB (Widget w, int item_no, XmAnyCallbackStruct *call_data)
             if (dialog == NULL) {
               num = 0;
               XtSetArg ( args[num], XmNautoUnmanage, False); num++;
-              dialog = XmCreateSelectionDialog( Toplevel, "Accountsel",
+              dialog = XmCreateSelectionDialog( per->Toplevel, "Accountsel",
                                                 args, num);
 
               /* I don't need these */
@@ -899,11 +955,11 @@ void accountCB (Widget w, int item_no, XmAnyCallbackStruct *call_data)
     case 10:
     case 11:
     case 12:  /* Display account 9 */
+    default:
             setAccountMenuItem ( item_no - 3);
             displayAccount (item_no - 3);
             break;
 
-    default:  break;
 
   }
 
@@ -943,6 +999,7 @@ static void setAccountMenuItem(int which)
 static void makeAccountMenu()
 {
 
+  extern AppData appdata;
   extern Widget Accountmenu;
   WidgetList button;
   Widget newbutton;
@@ -959,7 +1016,7 @@ static void makeAccountMenu()
     XtDestroyWidget ( button[item] );
 
   /* Make the new variable menu items */
-  for ( item = 1; item <= accountno && item < 10; item++) {
+  for ( item = 1; item <= accountno && item < appdata.maxaccounts; item++) {
     name = createAccountName ( item );
     if (name == NULL) {
       write_status ("Out of memory: could not create account menu.", ERR);
@@ -976,7 +1033,7 @@ static void makeAccountMenu()
     XtFree (name);
   }
 
-  if (accountno > 9) {
+  if (accountno >= appdata.maxaccounts) {
     newbutton = XtVaCreateManagedWidget ( "more",
                           xmPushButtonGadgetClass, Accountmenu,
                           NULL );
@@ -1031,11 +1088,14 @@ void saveAccount ( FILE *savefile, int index)
               account[index].calc_nav->year, account[index].calc_nav->value );
 }
 
-double accountApplyModifier (char *modifier, double nav) {
+double accountApplyModifier (char *modifier, double nav)
+{
   char *next, *end, *end_opnd;
   char err_msg[60];
   double result = nav, opnd;
-
+#if 0
+  fprintf(stderr, "accountApplyModifier(%s,%f)\n", modifier, nav);
+#endif
   if (modifier == NULL)
     return (nav);
 
@@ -1078,6 +1138,9 @@ double accountApplyModifier (char *modifier, double nav) {
   return (result);
 }
 
+/*
+ * This is only used from the remote interface (read: from Xquote).
+ */
 int accountMatchTicker (char *ticker, int month, int day, int year, 
                         double nav)
 {
@@ -1093,8 +1156,8 @@ int accountMatchTicker (char *ticker, int month, int day, int year,
       account[accountno].calc_nav->date = day;
       account[accountno].calc_nav->year = year;
       getAccount (accountno, ACCOUNT_ETICKER, &acct_ticker);
-      nav = accountApplyModifier (acct_ticker, nav);
-      account[accountno].calc_nav->value = nav;
+      account[accountno].calc_nav->value = accountApplyModifier (acct_ticker, nav);
+      account[accountno].flags |= ACCOUNT_LIVEUPDATE;
     }
     /* Account Exchange Rate */
     getAccount ( accountno, ACCOUNT_CTICKER, &acct_ticker);
@@ -1104,8 +1167,8 @@ int accountMatchTicker (char *ticker, int month, int day, int year,
       account[accountno].curr_nav->date = day;
       account[accountno].curr_nav->year = year;
       getAccount (accountno, ACCOUNT_ECTICKER, &acct_ticker);
-      nav = accountApplyModifier (acct_ticker, nav);
-      account[accountno].curr_nav->value = nav;
+      account[accountno].curr_nav->value = accountApplyModifier (acct_ticker, nav);
+      account[accountno].flags |= ACCOUNT_LIVEUPDATE;
     }
   }
   return ((found)?accountno:0);
@@ -1143,3 +1206,42 @@ void accountHighlightDay (char *date, int account, Widget w)
       break;
     }
 }
+
+int Account2CurrencyAccount(int a)
+{
+	if (! a)
+		return 0;
+	if (account[a].currency_account)
+		return account[a].currency_account;
+	return 0;
+}
+
+char *GetAccountTitle(int a)
+{
+	return account[a].title;
+}
+
+double GetCurrencyForDate(int num, long ldate)
+{
+	double	r;
+	int	i;
+
+	if (account[num].trans[1].t_nav == 0.0) {
+		fprintf(stderr, "GetCurrencyForDate => 0\n");
+		return 0.;
+	} else
+		r = account[num].trans[1].t_nav;
+
+#if 0
+	fprintf(stderr, "GetCurrencyForDate(%s, %ld)\n", account[num].title, ldate);
+#endif
+	for (i=1; i<=account[num].transno; i++) {
+		if (account[num].trans[i].t_ldate == ldate)
+			return account[num].trans[i].t_nav;
+		else if (account[num].trans[i].t_ldate < ldate)
+			r = account[num].trans[i].t_nav;
+		else
+			return r;
+	}
+	return r;
+}
