@@ -1,28 +1,35 @@
 /*
-  Xinvest Copyright (c)1995-98 Mark Buser,
-  All Rights Reserved.
-
-  Permission is hereby granted to copy
-  and freely distribute copies of this
-  program for non-commercial purposes
-  without fee, provided that this notice
-  appears in all copies.
-  
-  All redistributions must be in their
-  entirety as originally distributed.
-  Feel free to modify Xinvest, but
-  modified versions may not be distributed
-  without prior consent of the author.
-  
-  This software is provided 'as-is'
-  without any express or implied warranty.
-  In no event will the author be held
-  liable for any damages resulting from
-  the use of this software.
-
-  $Revision: 2.12 $ $Date: 2000/01/25 22:46:13 $
-*/
-
+ * Copyright (c) 1996, 1997, 1998, 2000, Mark Buser.
+ * Copyright (c) 2003, 2004, Danny Backx.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are
+ * met:
+ *
+ * Redistributions of source code must retain the above copyright notice,
+ * this list of conditions and the following disclaimer.
+ * Redistributions in binary form must reproduce the above copyright
+ * notice, this list of conditions and the following disclaimer in the
+ * documentation and/or other materials provided with the distribution.
+ * Neither the names the authors (see above), nor the names of other
+ * contributors may be used to endorse or promote products derived from
+ * this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * $Header: /pack/anoncvs/xinvest/src/account.h,v 2.17 2004/04/18 10:58:36 danny Exp $
+ */
 int activeAccount();              /* Return index number of current account */
 int numAccounts();                 /* Return number of open accounts */
 
@@ -33,6 +40,7 @@ int numAccounts();                 /* Return number of open accounts */
 #define ACCOUNT_UPDATE        2   /* Changes need to be processed */
 #define ACCOUNT_IN_PORTFOLIO  4   /* Account has been added  */
 #define ACCOUNT_WRITABLE      8   /* Account file is writable  */
+#define ACCOUNT_LIVEUPDATE   16   /* We've had a live-update for this account */
 
 int accountStatus(int which, unsigned flag ); /* Is 'flag' set in 'which' */
 int accountsStatus(int flag );    /* Is flag set anywhere */
@@ -103,3 +111,7 @@ int accountMatchTicker (char *, int, int, int, double);
 
 /* Change background of Widget if passed date matches one in transaction */
 void accountHighlightDay (char *, int, Widget);
+void ProcessAccountsForCticker(void);
+int Account2CurrencyAccount(int);
+char *GetAccountTitle(int);
+double GetCurrencyForDate(int num, long ldate);
