@@ -1,28 +1,35 @@
 /*
-  Xinvest Copyright (c)1995-98 Mark Buser,
-  All Rights Reserved.
-
-  Permission is hereby granted to copy
-  and freely distribute copies of this
-  program for non-commercial purposes
-  without fee, provided that this notice
-  appears in all copies.
-  
-  All redistributions must be in their
-  entirety as originally distributed.
-  Feel free to modify Xinvest, but
-  modified versions may not be distributed
-  without prior consent of the author.
-  
-  This software is provided 'as-is'
-  without any express or implied warranty.
-  In no event will the author be held
-  liable for any damages resulting from
-  the use of this software.
-
-  $Revision: 2.11 $ $Date: 1998/05/02 15:56:52 $
-*/
-
+ * Copyright (c) 1995, 1996, 1997, 1998, Mark Buser.
+ * Copyright (c) 2001, 2003, Danny Backx.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are
+ * met:
+ *
+ * Redistributions of source code must retain the above copyright notice,
+ * this list of conditions and the following disclaimer.
+ * Redistributions in binary form must reproduce the above copyright
+ * notice, this list of conditions and the following disclaimer in the
+ * documentation and/or other materials provided with the distribution.
+ * Neither the names the authors (see above), nor the names of other
+ * contributors may be used to endorse or promote products derived from
+ * this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * $Header: /pack/anoncvs/xinvest/src/functions.c,v 2.15 2003/11/19 19:35:10 danny Exp $
+ */
 #include <stdlib.h>
 #include <ctype.h>
 #include <math.h>
@@ -37,6 +44,8 @@
 #include "util.h"
 #include "xutil.h"
 
+#include "xinvest.h"
+
 #define PV   0
 #define FV   1
 #define RATE 2
@@ -133,7 +142,6 @@ void draw_funccalc ( Widget Drawing, Pixmap Drawpixmap, GC gc)
 
   Dimension width, height;
   Pixel foreground, background;
-  extern XFontStruct *large;
   int x, y; 
   int boxX, boxWidth;
 
@@ -185,7 +193,7 @@ void draw_funccalc ( Widget Drawing, Pixmap Drawpixmap, GC gc)
   /* 
   ** How many lines of text 
   */
-  XSetFont ( XtDisplay (Drawing), gc, large->fid );
+  XSetFont ( XtDisplay (Drawing), gc, per->large->fid );
 
   /* How many chars will fit in 95% of the box width with word wrap. */
   lines = 0;
@@ -193,10 +201,10 @@ void draw_funccalc ( Widget Drawing, Pixmap Drawpixmap, GC gc)
   /* Title */
   nextchar = title[curr_func];
   numchars = strlen(nextchar);
-  textWidth = XTextWidth ( large, nextchar, numchars);
+  textWidth = XTextWidth ( per->large, nextchar, numchars);
   while ( (numchars && !isspace( nextchar[numchars] ) ) || 
           textWidth > 0.95*boxWidth )
-    textWidth = XTextWidth ( large, nextchar, numchars-- );
+    textWidth = XTextWidth ( per->large, nextchar, numchars-- );
   lineChars = (int *) XtRealloc ( (char *)lineChars, sizeof(int)*(lines+1));
   lineChars[lines++] = numchars;
 
@@ -205,10 +213,10 @@ void draw_funccalc ( Widget Drawing, Pixmap Drawpixmap, GC gc)
   numchars = strlen(nextchar);
 
   while ( numchars ) {
-    textWidth = XTextWidth ( large, nextchar, numchars);
+    textWidth = XTextWidth ( per->large, nextchar, numchars);
     while ( (numchars && !isspace( nextchar[numchars] ) ) || 
             textWidth > 0.95*boxWidth )
-      textWidth = XTextWidth ( large, nextchar, numchars-- );
+      textWidth = XTextWidth ( per->large, nextchar, numchars-- );
 
     lineChars = (int *) XtRealloc ( (char *)lineChars, sizeof(int)*(lines+1));
     lineChars[lines++] = numchars;
@@ -216,7 +224,7 @@ void draw_funccalc ( Widget Drawing, Pixmap Drawpixmap, GC gc)
     nextchar += (numchars +1);
     numchars = strlen(nextchar);
   }
-  textHeight = large->ascent + large->descent;
+  textHeight = per->large->ascent + per->large->descent;
 
   /* Where to draw the box */
   x = boxX;
@@ -240,7 +248,7 @@ void draw_funccalc ( Widget Drawing, Pixmap Drawpixmap, GC gc)
     else
       nextchar += lineChars[curLine-1] +1;
 
-    textWidth = XTextWidth ( large, nextchar, lineChars[curLine] );
+    textWidth = XTextWidth ( per->large, nextchar, lineChars[curLine] );
     /* Center this line in box */
     x = (boxWidth - textWidth) /2 + boxX;
 
@@ -314,7 +322,7 @@ void draw_funccalc ( Widget Drawing, Pixmap Drawpixmap, GC gc)
         XSetForeground ( XtDisplay(Drawing), gc, background );
         XSetBackground ( XtDisplay(Drawing), gc, foreground );
         XDrawImageString ( XtDisplay (Drawing), Drawpixmap, gc,
-                         x, y + large->ascent, nextchar+i, 1 );
+                         x, y + per->large->ascent, nextchar+i, 1 );
         XSetForeground ( XtDisplay(Drawing), gc, foreground );
         XSetBackground ( XtDisplay (Drawing), gc, background );
       } else {
@@ -322,18 +330,18 @@ void draw_funccalc ( Widget Drawing, Pixmap Drawpixmap, GC gc)
         if (colorVisual && curcolor != bot_shadow) {
           XSetForeground ( XtDisplay (Drawing), gc, bot_shadow );
           XDrawString ( XtDisplay (Drawing), Drawpixmap, gc,
-                        x +1, y + large->ascent +1, nextchar+i, 1 );
+                        x +1, y + per->large->ascent +1, nextchar+i, 1 );
         }
         XSetForeground ( XtDisplay (Drawing), gc, curcolor );
         XDrawString ( XtDisplay (Drawing), Drawpixmap, gc,
-                      x, y + large->ascent, nextchar+i, 1 );
+                      x, y + per->large->ascent, nextchar+i, 1 );
       }
 
       /* There may not be a space after a digit */
       if (i == numchars-1 && indigit) 
         numnumbers++;
 
-      x += XTextWidth ( large, nextchar+i, 1);
+      x += XTextWidth ( per->large, nextchar+i, 1);
     }
 
   }
@@ -476,7 +484,7 @@ void calcFuncSel ( Widget w, int client_data, XmAnyCallbackStruct *call_data )
                                       XmNlabelString, terms[PRDS], 
                                       NULL );
                       XtVaSetValues ( Calclabel, 
-                                      XmNlabelPixmap, GetPixmap(PFV, NORMAL),
+                                      XmNlabelPixmap, GetPixmap(PFV, NORMAL, XtDisplay(Calclabel)),
                                       NULL);
                       break;
     case FUNC_FVACC:  /* Future value of series of payments */
@@ -491,7 +499,7 @@ void calcFuncSel ( Widget w, int client_data, XmAnyCallbackStruct *call_data )
                                       XmNlabelString, terms[PRDS], 
                                       NULL );
                       XtVaSetValues ( Calclabel, 
-                                      XmNlabelPixmap, GetPixmap(PSFV, NORMAL),
+                                      XmNlabelPixmap, GetPixmap(PSFV, NORMAL, XtDisplay(Calclabel)),
                                       NULL);
                       break;
     case FUNC_PERPAY: /* Amount of the periodic payment of a loan */
@@ -506,7 +514,7 @@ void calcFuncSel ( Widget w, int client_data, XmAnyCallbackStruct *call_data )
                                       XmNlabelString, terms[PRDS], 
                                       NULL );
                       XtVaSetValues ( Calclabel, 
-                                      XmNlabelPixmap, GetPixmap(PPER, NORMAL),
+                                      XmNlabelPixmap, GetPixmap(PPER, NORMAL, XtDisplay(Calclabel)),
                                       NULL);
                       break;
     case FUNC_PV:     /* Present value of series of payments */
@@ -521,7 +529,7 @@ void calcFuncSel ( Widget w, int client_data, XmAnyCallbackStruct *call_data )
                                       XmNlabelString, terms[PRDS], 
                                       NULL );
                       XtVaSetValues ( Calclabel, 
-                                      XmNlabelPixmap, GetPixmap(PPV, NORMAL),
+                                      XmNlabelPixmap, GetPixmap(PPV, NORMAL, XtDisplay(Calclabel)),
                                       NULL);
                       break;
     case FUNC_RATE:   /* Periodic interest rate for PV to reach FV */
@@ -536,7 +544,7 @@ void calcFuncSel ( Widget w, int client_data, XmAnyCallbackStruct *call_data )
                                       XmNlabelString, terms[PRDS], 
                                       NULL );
                       XtVaSetValues ( Calclabel, 
-                                      XmNlabelPixmap, GetPixmap (PRATE, NORMAL),
+                                      XmNlabelPixmap, GetPixmap (PRATE, NORMAL, XtDisplay(Calclabel)),
                                       NULL);
                       break;
   }
