@@ -1,28 +1,35 @@
 /*
-  Xinvest Copyright (c)1995-99, 2000
-  Mark Buser, All Rights Reserved.
-
-  Permission is hereby granted to copy
-  and freely distribute copies of this
-  program for non-commercial purposes
-  without fee, provided that this notice
-  appears in all copies.
-  
-  All redistributions must be in their
-  entirety as originally distributed.
-  Feel free to modify Xinvest, but
-  modified versions may not be distributed
-  without prior consent of the author.
-  
-  This software is provided 'as-is'
-  without any express or implied warranty.
-  In no event will the author be held
-  liable for any damages resulting from
-  the use of this software.
-
-  $Revision: 2.41 $ $Date: 2000/02/04 00:42:58 $
-*/
-
+ * Copyright (c) 1994, 1995, 1996, 1997, 1998, 1999, 2000, Mark Buser.
+ * Copyright (c) 2001, 2003, 2004, Danny Backx.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are
+ * met:
+ *
+ * Redistributions of source code must retain the above copyright notice,
+ * this list of conditions and the following disclaimer.
+ * Redistributions in binary form must reproduce the above copyright
+ * notice, this list of conditions and the following disclaimer in the
+ * documentation and/or other materials provided with the distribution.
+ * Neither the names the authors (see above), nor the names of other
+ * contributors may be used to endorse or promote products derived from
+ * this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * $Header: /pack/anoncvs/xinvest/src/graph.c,v 2.47 2004/04/24 07:58:40 danny Exp $
+ */
 #include <float.h>
 #include <limits.h>
 #include <stdio.h>
@@ -48,8 +55,10 @@
 #include "util.h"
 #include "xutil.h"
 
-#include "pixmaps/cumstipple.xbm"
-#include "pixmaps/movstipple.xbm"
+#include "xinvest.h"
+
+#include "pixmap/cumstipple.xbm"
+#include "pixmap/movstipple.xbm"
 
 
 static int multiAccount = 0;
@@ -851,17 +860,17 @@ static struct trans_value *calc_trans_values( int account, int transno,
  trans_date = GetTransLDate(transactions, transno);
 
  if (trans_date == GetTransLDate (transactions, numTrans)) {
-   trans.value[PRICE-1] = rate_price (account, CURRENT_DATE);
+   trans.value[PRICE-1] = rate_price (account, CURRENT_DATE, TRUE);
    trans.value[VALUE-1] = rate_value (account, trans_date, CURRENT_DATE);
    trans.value[VALUE_CUM-1] = rate_value (account, FIRST_DATE, CURRENT_DATE);
  } else {
-   trans.value[PRICE-1] = rate_price (account, trans_date);
+   trans.value[PRICE-1] = rate_price (account, trans_date, TRUE);
    trans.value[VALUE-1] = rate_value (account, trans_date, trans_date);
    trans.value[VALUE_CUM-1] = rate_value (account, FIRST_DATE, trans_date);
  }
  /* This form allows us to show splits as an increase in shares */
- trans.value[SHARE-1] = rate_shares (account, FIRST_DATE, trans_date) -
-	                rate_shares (account, FIRST_DATE, trans_date-1);
+ trans.value[SHARE-1] = rate_shares (account, FIRST_DATE, trans_date, TRUE) -
+	                rate_shares (account, FIRST_DATE, trans_date-1, TRUE);
  trans.value[COST-1]  = rate_cost (account, trans_date, trans_date);
  { 
    YIELD result;
@@ -870,7 +879,7 @@ static struct trans_value *calc_trans_values( int account, int transno,
  }
 
 
- trans.value[SHARE_CUM-1] = rate_shares (account, FIRST_DATE, trans_date);
+ trans.value[SHARE_CUM-1] = rate_shares (account, FIRST_DATE, trans_date, TRUE);
  trans.value[COST_CUM-1]  = rate_cost (account, FIRST_DATE, trans_date);
  { 
    YIELD result;
@@ -925,13 +934,13 @@ static struct trans_value *calc_trans_values( int account, int transno,
        /* On last transaction pick up any price points */
        if (eDate == trans_date && 
 	   trans_date == GetTransLDate (transactions, numTrans))
-         trans.value[PRICE_MAVG-1] += rate_price (account, CURRENT_DATE);
+         trans.value[PRICE_MAVG-1] += rate_price (account, CURRENT_DATE, TRUE);
        else
-         trans.value[PRICE_MAVG-1] += rate_price (account, eDate);
+         trans.value[PRICE_MAVG-1] += rate_price (account, eDate, TRUE);
      }
      trans.value[PRICE_MAVG-1] /= (trans_date-sDate);
 
-     trans.value[SHARE_MAVG-1] = rate_shares (account, sDate, trans_date);
+     trans.value[SHARE_MAVG-1] = rate_shares (account, sDate, trans_date, TRUE);
      trans.value[COST_MAVG-1] = rate_cost (account, sDate, trans_date);
      trans.value[DISTRIB_MAVG-1] = rate_distrib (account, sDate, trans_date);
 
@@ -1006,7 +1015,6 @@ void plot ( Widget Drawing, Pixmap Drawpixmap, GC gc )
 
   XPoint *pnts = NULL;
   Pixel  savefore, saveback;
-  extern XFontStruct *small;
 
   int account = activeAccount();
   PLOT_VAR *graphVar;
@@ -1025,9 +1033,9 @@ void plot ( Widget Drawing, Pixmap Drawpixmap, GC gc )
                           XmNwidth,      &plot_attrib.width,
                  NULL );
 
-  x_border = XTextWidth (small, "9999999", strlen("9999999")) +5;
-  y_border = small->ascent + small->descent +5;
-  t_border = small->ascent + small->descent;
+  x_border = XTextWidth (per->small, "9999999", strlen("9999999")) +5;
+  y_border = per->small->ascent + per->small->descent +5;
+  t_border = per->small->ascent + per->small->descent;
 
   /* Don't plot on horizontal ticks */
   plot_attrib.height -= y_border;  
@@ -1253,9 +1261,11 @@ void plot ( Widget Drawing, Pixmap Drawpixmap, GC gc )
   /* This doesn't match the other plot types so is handled separately */ 
   if ( PLOT_ACTIVE( CONST ) ) {   /* constant at fixed rate */
 
+#if 0
     static Widget wstart = (Widget) NULL; 
     static Widget wrate = (Widget) NULL;
     static GC const_gc = NULL;
+#endif
     XRectangle plot_clip;
     PLOT_VAR *accountGraph;
 
@@ -1268,17 +1278,17 @@ void plot ( Widget Drawing, Pixmap Drawpixmap, GC gc )
     long graphdays = GetTransLDate(trans, numTrans) - GetTransLDate(trans, 1);
 
     /* Find our widget names from the parent */
-    if (wstart == NULL || wrate == NULL) {
-      wstart = XtNameToWidget ( Graphcform, "Graphvstart");
-      wrate = XtNameToWidget ( Graphcform, "Graphvrate");
+    if (per->wstart == NULL || per->wrate == NULL) {
+      per->wstart = XtNameToWidget ( Graphcform, "Graphvstart");
+      per->wrate = XtNameToWidget ( Graphcform, "Graphvrate");
     }
 
     /* Get user selection. Checked in text callback. */
-    str = XmTextGetString ( wstart );
+    str = XmTextGetString ( per->wstart );
     start = strtod ( str, NULL ) * plot_attrib.scale;
     XtFree ( str );
 
-    str = XmTextGetString ( wrate );
+    str = XmTextGetString ( per->wrate );
     rate = strtod ( str, NULL );           /* Checked in text callback */
     XtFree ( str );
 
@@ -1293,12 +1303,12 @@ void plot ( Widget Drawing, Pixmap Drawpixmap, GC gc )
         (end - (float)plot_attrib.min) / (float)plot_attrib.range;
   
     /* Make a GC for setting the constant line clip area */
-    if (const_gc == NULL ) 
-       const_gc = XCreateGC ( XtDisplay(Drawing), 
+    if (per->const_gc == NULL ) 
+       per->const_gc = XCreateGC ( XtDisplay(Drawing), 
                               RootWindowOfScreen( XtScreen(Drawing)), 0, 0);
     /* Get the plot drawing attribute. */
     accountGraph = graph_attrib ( activeAccount() );
-    SetDrawAttrib ( Drawing, const_gc, accountGraph->attrib[CONST-1] );
+    SetDrawAttrib ( Drawing, per->const_gc, accountGraph->attrib[CONST-1] );
 
     /* Do these values cross the plot range? */
     if (!( (start > plot_attrib.max && end > plot_attrib.max) ||
@@ -1310,10 +1320,10 @@ void plot ( Widget Drawing, Pixmap Drawpixmap, GC gc )
       plot_clip.y = (short) t_border;
       plot_clip.width = (short) plot_attrib.width;
       plot_clip.height = (short) plot_attrib.height - t_border;
-      XSetClipRectangles ( XtDisplay(Drawing), const_gc, 
+      XSetClipRectangles ( XtDisplay(Drawing), per->const_gc, 
                            0, 0, &plot_clip, 1, Unsorted );
 
-      XDrawLine ( XtDisplay(Drawing), Drawpixmap, const_gc, 
+      XDrawLine ( XtDisplay(Drawing), Drawpixmap, per->const_gc, 
                   x_border, plot_attrib.height - start_y, 
                   x, plot_attrib.height - y );
     }
@@ -1336,10 +1346,15 @@ void plot ( Widget Drawing, Pixmap Drawpixmap, GC gc )
 
     Pixel bot_shadow;
 
-    if ( isColorVisual( XtDisplay(Drawing) ) )
-      XmGetColors ( XtScreen (Drawing), GetColormap(), saveback, 
+    if (isColorVisual(XtDisplay(Drawing))) {
+      /* FIX ME */
+      if (GetColormap() == 0)
+	XmGetColors(XtScreen (Drawing), DefaultColormapOfScreen(XtScreen(Drawing)), saveback, 
                     NULL, NULL, &bot_shadow, NULL );
-    else
+      else
+	XmGetColors(XtScreen (Drawing), GetColormap(), saveback, 
+                    NULL, NULL, &bot_shadow, NULL );
+    } else
       bot_shadow = saveback;
 
     title = createAccountName ( activeAccount() );
@@ -1355,9 +1370,9 @@ void plot ( Widget Drawing, Pixmap Drawpixmap, GC gc )
              (title)?title:NULL ); 
     XtFree (title);
 
-    XSetFont ( XtDisplay (Drawing), gc, small->fid );
-    str_height = small->ascent +1;
-    str_width = XTextWidth ( small, status, strlen(status) );
+    XSetFont ( XtDisplay (Drawing), gc, per->small->fid );
+    str_height = per->small->ascent +1;
+    str_width = XTextWidth ( per->small, status, strlen(status) );
 
     if ( bound_ok ( str_width +5, plot_attrib.width, &errors, 0) ) {
        XSetForeground( XtDisplay(Drawing), gc, bot_shadow );
@@ -1395,12 +1410,12 @@ void plot ( Widget Drawing, Pixmap Drawpixmap, GC gc )
       /* Center text around the tick mark */
       XSetForeground( XtDisplay(Drawing), gc, bot_shadow );
       XDrawString( XtDisplay(Drawing), Drawpixmap, gc,
-                   x_border - XTextWidth( small, tick, strlen(tick)) -5 +1,
+                   x_border - XTextWidth( per->small, tick, strlen(tick)) -5 +1,
                    y +1,
                    tick, strlen(tick) );
       XSetForeground( XtDisplay(Drawing), gc, savefore );
       XDrawString( XtDisplay(Drawing), Drawpixmap, gc,
-                   x_border - XTextWidth( small, tick, strlen(tick)) -5,
+                   x_border - XTextWidth( per->small, tick, strlen(tick)) -5,
                    y,
                    tick, strlen(tick) );
 
@@ -1429,7 +1444,7 @@ void plot ( Widget Drawing, Pixmap Drawpixmap, GC gc )
     } else {
       /* Make sure a formatted date fits */
       strcpy (tick, utilDateOut(plot_attrib.daymin));
-      div = plot_attrib.width / (XTextWidth (small, tick, strlen(tick)) +5);
+      div = plot_attrib.width / (XTextWidth (per->small, tick, strlen(tick)) +5);
       if ( plot_attrib.dayrange < div)
         div = plot_attrib.dayrange;
       x = x_border;
@@ -1503,8 +1518,8 @@ void plot ( Widget Drawing, Pixmap Drawpixmap, GC gc )
     /* If anything went awry, pop-up a message */
     /* We could get very specific, since we know where it went wrong */
     /* but don't right now. */
-    if ( errors && !suppressed() )
-      write_status ("Title too large, increase graph size or\n"
+    if (errors && per->Toplevel && !suppressed())
+      write_status("Title too large, increase graph size or\n"
                     "reduce font sizes. See \"help Fonts\".", WARN ); 
   }
 
