@@ -1,28 +1,35 @@
 /*
-  Xinvest Copyright (c)1995-99 Mark Buser,
-  All Rights Reserved.
-
-  Permission is hereby granted to copy
-  and freely distribute copies of this
-  program for non-commercial purposes
-  without fee, provided that this notice
-  appears in all copies.
-  
-  All redistributions must be in their
-  entirety as originally distributed.
-  Feel free to modify Xinvest, but
-  modified versions may not be distributed
-  without prior consent of the author.
-  
-  This software is provided 'as-is'
-  without any express or implied warranty.
-  In no event will the author be held
-  liable for any damages resulting from
-  the use of this software.
-
-  $Revision: 2.25 $ $Date: 2000/01/25 22:47:40 $
-*/
-
+ * Copyright (c) 1995, 1996, 1997, 1998, 1999, 2000, 2001, Mark Buser.
+ * Copyright (c) 2001, 2003, Danny Backx.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are
+ * met:
+ *
+ * Redistributions of source code must retain the above copyright notice,
+ * this list of conditions and the following disclaimer.
+ * Redistributions in binary form must reproduce the above copyright
+ * notice, this list of conditions and the following disclaimer in the
+ * documentation and/or other materials provided with the distribution.
+ * Neither the names the authors (see above), nor the names of other
+ * contributors may be used to endorse or promote products derived from
+ * this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * $Header: /pack/anoncvs/xinvest/src/portfolio.c,v 2.28 2003/11/19 19:35:11 danny Exp $
+ */
 #include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -47,6 +54,8 @@
 #include "util.h"
 #include "xutil.h"
 
+#include "xinvest.h"
+
 static NODE root  = { "Root", -1, False, NULL, NULL };
 static NODE funds = { "Accounts", -1, False, NULL, NULL };
 static NODE all   = { "Flatten", -1, False, NULL, NULL };
@@ -54,7 +63,6 @@ static double total_value = 0.0;
 
 /* forward declarations */
 static double what_value ( NODE * );
-static double extract_percent ( char * );
 static char  *mknodes ( NODE *, char *, float, int);
 static char  *delnodes ( NODE *, char *, float, int );
 static int   exists ( NODE *, char *);
@@ -235,6 +243,10 @@ void portAccount( int account, int what )
   title = createAccountName (account);
   getAccount ( account, ACCOUNT_TRANSTEXT, &text);
   if ( text ) {
+
+    double thisPercent = 0.0;
+    char thisCategory[80] = "";
+
     /* This is destructive to text, so copy it */
     text = strdup (text);
  
@@ -249,54 +261,104 @@ void portAccount( int account, int what )
       if (rear)
         *rear = '\0';
 
-      /* 
-      ** Update root node
-      */
-      if ( strncasecmp ( "percent", front, 7 ) == 0 ) {
-
-        all_percents += extract_percent ( front );
-
-        /* make dummy percents line */
-        strcpy ( dummy_str, front );
-
-        /* true, report errors */
-        if (what == PORT_ADD) 
-          node_error = mknodes ( &root, front, value, True );
-        else 
-          node_error = delnodes ( &root, front, value, True );
-       
-        if (node_error) {
-          sprintf ( errmsg, "%s:\n%s", title, node_error);
-          write_status (errmsg, ERR);
-        }
+      if (strncasecmp ( "percent", front, 7 ) == 0) {
+        int num_read = 0, error = 0;
+                                   /* Percent percent% category[.category] */
+        num_read = sscanf ( front, "%*s %lf%% %s", &thisPercent, thisCategory );
 
         /*
-        ** Update all node 
+        ** Syntax error checks report only on add
         */
-        front = dummy_str;
-
-        /* skip to percent digits */
-        while ( !isdigit (*front) )  
-          front++;
-        /* skip over percent digits */
-        while ( isdigit (*front) || ispunct(*front) )  
-          front++;
-
-        /* remove '.' from elements only */
-        while ( *front != '\0' ) { 
-          if ( *front == '.' )
-            *front = '_';
-          front++;
+        /* Check num parameters */
+        if (num_read != 2) {
+          /* Check for missing '%' */
+          if (strchr (front, '%') == NULL) {
+            sprintf ( errmsg, "Missing '%%', expected ddd.dd%% in\n'%s'\n", 
+                      front);
+          } else {
+            sprintf ( errmsg, "Malformed, wrong order, or wrong number of\n"
+                              "parameters in '%s', parsed %d.\n", 
+                              front, num_read);
+          }
+          if (what == PORT_ADD) 
+            write_status (errmsg, ERR);
+          error = 1;
+        }
+        /* Check percent value */
+        else if (thisPercent <= 0 || thisPercent > 100) {
+          sprintf ( errmsg, "Unexpected Percent value: %f,\n"
+                            "valid range is 0 < percent <= 100.\n", 
+                            thisPercent);
+          if (what == PORT_ADD) 
+            write_status (errmsg, ERR);
+          error = 1;
+        } else {
+          /* Check category for valid characters */
+          for (num_read=0; num_read < strlen(thisCategory); num_read++) {
+            if ( !(isalnum (thisCategory[num_read]) || 
+                  thisCategory[num_read] == '.') ) {
+              sprintf (errmsg, "Unexpected character value: '%c',\nin '%s'\n",
+                       thisCategory[num_read], thisCategory);
+              if (what == PORT_ADD) 
+                write_status (errmsg, ERR);
+              error = 1;
+              break;
+            }  
+          }
         }
 
-        /* False, skip errors caught above */
-        if (what == PORT_ADD)
-          mknodes ( &all, dummy_str, value, False );
-        else
-          delnodes ( &all, dummy_str, value, False ); 
-      }
-      front = rear +1;
-    }
+        if (!error) {
+          /* 
+          ** Update root node
+          */
+          all_percents += thisPercent;
+
+          /* make dummy percents line */
+          strcpy ( dummy_str, front );
+
+          /* true, report errors */
+          if (what == PORT_ADD) 
+            node_error = mknodes ( &root, front, value, True );
+          else 
+            node_error = delnodes ( &root, front, value, True );
+
+          if (node_error) {
+            sprintf ( errmsg, "%s:\n%s", title, node_error);
+            write_status (errmsg, ERR);
+          }
+
+          /*
+          ** Update all node 
+          */
+          front = dummy_str;
+
+          /* skip to percent digits */
+          while ( !isdigit (*front) )  
+            front++;
+          /* skip over percent digits */
+          while ( isdigit (*front) || ispunct(*front) )  
+            front++;
+
+          /* remove '.' from elements only */
+          while ( *front != '\0' ) { 
+            if ( *front == '.' )
+              *front = '_';
+            front++;
+          }
+
+          /* False, skip errors caught above */
+          if (what == PORT_ADD)
+            mknodes ( &all, dummy_str, value, False );
+          else
+            delnodes ( &all, dummy_str, value, False ); 
+
+        } /* if no error */
+
+      } /* if percent line */
+
+      front = rear +1;   /* next line */
+
+    } /* while percent lines in account */
 
     if ( what == PORT_ADD) {
       if ( abs ((double)100.0 - all_percents) > 1.0E-6 ) { 
@@ -557,7 +619,6 @@ int pie ( Widget Drawing, Pixmap Drawpixmap, GC gc )
 
     int linex, liney;
  
-    extern XFontStruct *small, *large;
     char title[] = "Asset Allocation";
     int textWidth;
     int textHeight;
@@ -640,9 +701,9 @@ int pie ( Widget Drawing, Pixmap Drawpixmap, GC gc )
 
 
     /* Draw Title */
-    XSetFont ( XtDisplay (Drawing), gc, large->fid );
-    textWidth = XTextWidth ( large, title, strlen ( title ) );
-    textHeight = large->ascent + large->descent;
+    XSetFont ( XtDisplay (Drawing), gc, per->large->fid );
+    textWidth = XTextWidth ( per->large, title, strlen ( title ) );
+    textHeight = per->large->ascent + per->large->descent;
 
     x = (int)(width - textWidth)/2;
     if (bound_ok ( x+textWidth, width, &errors, count ) ) {
@@ -862,8 +923,8 @@ int pie ( Widget Drawing, Pixmap Drawpixmap, GC gc )
       /* Where does the text go? Calculate positions of maximum number of
       ** cells of text that can fit.
       */
-      cellheight = small->ascent+small->descent+large->ascent+large->descent;
-      cells = (height - (large->ascent + large->descent)) / cellheight;
+      cellheight = per->small->ascent+per->small->descent+per->large->ascent+per->large->descent;
+      cells = (height - (per->large->ascent + per->large->descent)) / cellheight;
 
       textLoc = (struct textTable *) XtCalloc (cells, sizeof(struct textTable));
       for ( i=0; i < cells; i++ ) {
@@ -1010,11 +1071,11 @@ int pie ( Widget Drawing, Pixmap Drawpixmap, GC gc )
 
         if (sliceLoc[i].slot != -1) {
           /* Draw the user supplied name */
-          XSetFont (XtDisplay (Drawing), gc, large->fid);
+          XSetFont (XtDisplay (Drawing), gc, per->large->fid);
           numchars = strlen (wedgeN);
-          textWidth = XTextWidth (large, wedgeN, numchars);
+          textWidth = XTextWidth (per->large, wedgeN, numchars);
           while (textWidth > (x - 20))
-            textWidth = XTextWidth (large, wedgeN, --numchars);
+            textWidth = XTextWidth (per->large, wedgeN, --numchars);
 
           if (sliceLoc[i].onleft)
             textx = textLoc[sliceLoc[i].slot].left_x;
@@ -1034,19 +1095,19 @@ int pie ( Widget Drawing, Pixmap Drawpixmap, GC gc )
                        textx, texty, wedgeN, numchars);
 
           /* Draw the percentage */
-          XSetFont (XtDisplay (Drawing), gc, small->fid);
+          XSetFont (XtDisplay (Drawing), gc, per->small->fid);
           numchars = strlen (wedgeP);
-          textWidth = XTextWidth (small, wedgeP, numchars);
+          textWidth = XTextWidth (per->small, wedgeP, numchars);
           while (textWidth > (x - 20))
-            textWidth = XTextWidth (small, wedgeP, --numchars);
+            textWidth = XTextWidth (per->small, wedgeP, --numchars);
 
           XSetForeground (XtDisplay(Drawing), gc, bot_shadow);
           XDrawString (XtDisplay (Drawing), Drawpixmap, gc,
-                       textx +1, texty + large->descent + small->ascent +1,
+                       textx +1, texty + per->large->descent + per->small->ascent +1,
                        wedgeP, numchars);
           XSetForeground ( XtDisplay(Drawing), gc, foreground);
           XDrawString (XtDisplay (Drawing), Drawpixmap, gc,
-                       textx, texty + large->descent + small->ascent,
+                       textx, texty + per->large->descent + per->small->ascent,
                        wedgeP, numchars);
         }
       }
@@ -1061,7 +1122,7 @@ int pie ( Widget Drawing, Pixmap Drawpixmap, GC gc )
     }
 
     /* Draw in some info on the total value */
-    XSetFont ( XtDisplay (Drawing), gc, small->fid );
+    XSetFont ( XtDisplay (Drawing), gc, per->small->fid );
     {
       char intro[] = "Value at '";
       char trail[] = "' - ";
@@ -1107,16 +1168,16 @@ int pie ( Widget Drawing, Pixmap Drawpixmap, GC gc )
       strcat (line, trail);
       strcat (line, value);
 
-      textWidth = XTextWidth ( small, line, strlen ( line ) );
+      textWidth = XTextWidth ( per->small, line, strlen ( line ) );
       x = (width - textWidth) / 2;
       if ( bound_ok ( x + textWidth, width, &errors, count) ||
 	   bound_ok ( 0, x, &errors, count) ) {
         XSetForeground ( XtDisplay(Drawing), gc, bot_shadow );
         XDrawString ( XtDisplay (Drawing), Drawpixmap, gc,
-                      x +1, height - small->descent +1, line, strlen(line) );
+                      x +1, height - per->small->descent +1, line, strlen(line) );
         XSetForeground ( XtDisplay(Drawing), gc, foreground );
         XDrawString ( XtDisplay (Drawing), Drawpixmap, gc,
-                      x, height - small->descent, line, strlen(line) );
+                      x, height - per->small->descent, line, strlen(line) );
       }
       XtFree (line);
     }
@@ -1461,16 +1522,3 @@ int getPActive ()
   XtVaGetValues ( Portpercentlist, XmNitemCount, &p_active, NULL );
   return (p_active);
 }
-
-static double  extract_percent ( char *line )
-{
-   char *next = line;
-
-   /* skip percent directive and white space */
-   while ( isalpha(*next) || isspace(*next) )
-     next++;
-
-   return ( atof ( next ) ); 
-
-}
-
