@@ -1,28 +1,35 @@
 /*
-  Xinvest Copyright (c)1996-1999, 2000 
-  Mark Buser, All Rights Reserved.
-
-  Permission is hereby granted to copy
-  and freely distribute copies of this
-  program for non-commercial purposes
-  without fee, provided that this notice
-  appears in all copies.
-
-  All redistributions must be in their
-  entirety as originally distributed.
-  Feel free to modify Xinvest, but
-  modified versions may not be distributed
-  without prior consent of the author.
-
-  This software is provided 'as-is'
-  without any express or implied warranty.
-  In no event will the author be held
-  liable for any damages resulting from
-  the use of this software.
-
-  $Revision: 2.6 $ $Date: 2000/01/31 22:13:43 $
-*/
-
+ * Copyright (c) 1998, 1999, 2000, Mark Buser.
+ * Copyright (c) 2001, 2003, 2004, Danny Backx.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are
+ * met:
+ *
+ * Redistributions of source code must retain the above copyright notice,
+ * this list of conditions and the following disclaimer.
+ * Redistributions in binary form must reproduce the above copyright
+ * notice, this list of conditions and the following disclaimer in the
+ * documentation and/or other materials provided with the distribution.
+ * Neither the names the authors (see above), nor the names of other
+ * contributors may be used to endorse or promote products derived from
+ * this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * $Header: /pack/anoncvs/xinvest/src/pref.c,v 2.13 2004/03/13 19:02:03 danny Exp $
+ */
 #include <stdlib.h>
 
 #include <Xm/XmAll.h>
@@ -33,24 +40,35 @@
 #include "pref.h"
 #include "xutil.h"
 
-/*
-** Externals
-*/
-extern Widget Toplevel;
+#ifndef	XQUOTE
+#ifndef	QUOTE_FILTER
+#define	XINVEST
+#endif
+#endif
+
+#ifdef	XINVEST
+#include "xinvest.h"
+#endif
 
 /*
 ** Local Globals for Preferences
 */
 
+static int  optLocCurr;
+static char *charLocCurr;
+static int  useExch;
+Widget	PrefLiveUpdateTF = NULL;
+int	PrefLiveUpdateInterval = -1;
+Boolean	PrefLiveUpdateSet = False;
+Widget	PrefLiveUpdateToggle = NULL;
+
+#ifdef	XINVEST
 /* Initialized in prefInitState() */
 static int  useBack;
-static int  useExch;
 static char *extBack;
 static int  optLocDate;
 static int  loptLocDate = 0;    /* set from opt menu callbacks */
-static int  optLocCurr;
 static int  loptLocCurr = 0;    /* set from opt menu callbacks */
-static char *charLocCurr;
 static int  prec;
 static int  lprec = 2;          /* set from opt menu callbacks */
 
@@ -62,12 +80,14 @@ static Widget optCurrChar = (Widget) NULL;
 static Widget optCurrExch = (Widget) NULL;
 static Widget optPrec = (Widget) NULL;
 
+static void prefLiveUpdateToggle(Widget w, XtPointer client, XtPointer call);
+
 /*
 ** Initialize preferences from resources
 */
 void prefInitState(char *appname)
 {
-  XrmDatabase   db = XtScreenDatabase(XtScreenOfObject(Toplevel));
+  XrmDatabase   db = XtScreenDatabase(XtScreenOfObject(per->Toplevel));
   XrmQuark      class[7], name[7];
   XrmValue      value;
   XrmRepresentation type;
@@ -200,6 +220,7 @@ static void prefPrecOpt (Widget w, XtPointer which, XtPointer call_data)
 */
 static void prefProc (Widget w, XtPointer which, XtPointer call_data)
 {
+	char	*p;
 
   switch ((int) which) {
 
@@ -244,7 +265,7 @@ static void prefProc (Widget w, XtPointer which, XtPointer call_data)
               if (num) {
 		char prompt[] = "Exchange rate preference change, "
 			        "update portfolio?"; 
-		if (AskUser(Toplevel, prompt, "OK", "Cancel", YES) == YES) {
+		if (AskUser(per->Toplevel, prompt, "OK", "Cancel", YES) == YES) {
 		  int i;
 		  /* Remove from portfolio */
 		  for (i=1; i <= numAccounts(); i++) {
@@ -267,6 +288,14 @@ static void prefProc (Widget w, XtPointer which, XtPointer call_data)
 	      XtFree ((char *)j);
             }
 
+	    /* Live update */
+	    p = XmTextFieldGetString(PrefLiveUpdateTF);
+	    if (p) {
+		    sscanf(p, "%d", &PrefLiveUpdateInterval);
+		    XtFree(p);
+	    }
+	    PrefLiveUpdateSet = XmToggleButtonGadgetGetState(PrefLiveUpdateToggle);
+
             /* Precision - set in own handler */
             prec = lprec;
             break;
@@ -327,7 +356,7 @@ int prefGetDate ()
 {
   return (optLocDate);
 }
-
+#endif	/* XINVEST */
 
 /* Set preferred currency format */
 void prefSetCurr (int which)
@@ -356,18 +385,6 @@ char *prefGetCurrChar ()
   return (charLocCurr);
 }
 
-/* Set arithmentic precision */
-void prefSetPrec (int which)
-{
-  if (which >= 0 && which <= 6)
-    prec = which;
-}
-
-int prefGetPrec ()
-{
-  return (prec);
-}
-
 void prefSetUseCurr (int use)
 {
   if (use)
@@ -381,12 +398,25 @@ int prefGetUseCurr ()
   return (useExch);
 }
 
+#ifdef	XINVEST
+/* Set arithmentic precision */
+void prefSetPrec (int which)
+{
+  if (which >= 0 && which <= 6)
+    prec = which;
+}
+
+int prefGetPrec ()
+{
+  return (prec);
+}
+
 
 /* Create preference dialog */
 static Widget prefCreatePreference()
 {
-  char *frames[] =  {"PrefBack", "PrefDate", "PrefCurr", "PrefPrec"};
-  char *forms[] =   {"form_0", "form_1", "form_2", "form_3"};
+  char *frames[] =  {"PrefBack", "PrefDate", "PrefCurr", "PrefPrec", "PrefLiveUpdate"};
+  char *forms[] =   {"form_0", "form_1", "form_2", "form_3", "form_4"};
   char *buttons[] = {"button_0", "button_1"};
 
 
@@ -396,7 +426,7 @@ static Widget prefCreatePreference()
   Dimension width, height, border;
 
   dialog = XtVaCreatePopupShell ("PrefPref", xmDialogShellWidgetClass,
-                                 Toplevel, NULL);
+                                 per->Toplevel, NULL);
 
   pane = XtVaCreateWidget ("PrefPane", xmPanedWindowWidgetClass,
                            dialog,
@@ -530,6 +560,49 @@ static Widget prefCreatePreference()
                 XtManageChild (optPrec);
                 break;
 
+	case 4:	/* Live Update Preferences */
+		{ Widget w;
+			char	s[8];
+		PrefLiveUpdateToggle = XtVaCreateManagedWidget("liveUpdateToggle",
+			xmToggleButtonGadgetClass, form,
+				XmNleftAttachment,	XmATTACH_FORM,
+				XmNrightAttachment,	XmATTACH_NONE,
+				XmNtopAttachment,	XmATTACH_FORM,
+				XmNbottomAttachment,	XmATTACH_NONE,
+			NULL);
+		XtAddCallback(PrefLiveUpdateToggle, XmNvalueChangedCallback,
+			prefLiveUpdateToggle, NULL);
+
+		w = XtVaCreateManagedWidget("liveUpdateLabel",
+			xmLabelGadgetClass, form,
+				XmNleftAttachment,	XmATTACH_FORM,
+				XmNleftOffset,		5,
+				XmNrightAttachment,	XmATTACH_NONE,
+				XmNtopAttachment,	XmATTACH_WIDGET,
+				XmNtopWidget,		PrefLiveUpdateToggle,
+				XmNtopOffset,		5,
+				XmNbottomAttachment,	XmATTACH_FORM,
+			NULL);
+                PrefLiveUpdateTF = XtVaCreateManagedWidget ("text_0",
+                            xmTextFieldWidgetClass, form,
+				XmNleftAttachment,	XmATTACH_WIDGET,
+				XmNleftWidget,		w,
+				XmNleftOffset,		5,
+				XmNrightAttachment,	XmATTACH_NONE,
+				XmNtopAttachment,	XmATTACH_WIDGET,
+				XmNtopWidget,		PrefLiveUpdateToggle,
+				XmNtopOffset,		5,
+				XmNbottomAttachment,	XmATTACH_FORM,
+			NULL);
+
+		/* Initialize */
+		XtSetSensitive(PrefLiveUpdateTF, PrefLiveUpdateSet);
+		sprintf(s, "%d", PrefLiveUpdateInterval);
+		XmTextFieldSetString(PrefLiveUpdateTF, s);
+		XmToggleButtonSetState(PrefLiveUpdateToggle, PrefLiveUpdateSet, False);
+		}
+		break;
+
      }
      XtManageChild (form);
 
@@ -586,3 +659,44 @@ void prefPostPreference()
   XtManageChild (dialog);
   XtPopup (dialog, XtGrabNone);
 }
+
+/*
+ * Handle the preferences for live update
+ */
+static void prefLiveUpdateToggle(Widget w, XtPointer client, XtPointer call)
+{
+	Boolean set = (((XmToggleButtonCallbackStruct *)call)->set == XmSET) ? True : False;
+	XtSetSensitive(PrefLiveUpdateTF, set);
+}
+#endif
+
+void prefSetLiveUpdateInterval(int i)
+{
+	static char	s[8];
+
+	if (i < 0) {
+		PrefLiveUpdateInterval = -i;
+		PrefLiveUpdateSet = False;
+	} else {
+		PrefLiveUpdateInterval = i;
+		PrefLiveUpdateSet = True;
+	}
+
+	if (PrefLiveUpdateTF) {
+		XtSetSensitive(PrefLiveUpdateTF, PrefLiveUpdateSet);
+		sprintf(s, "%d", i);
+		XmTextFieldSetString(PrefLiveUpdateTF, s);
+	}
+	if (PrefLiveUpdateToggle)
+		XmToggleButtonSetState(PrefLiveUpdateToggle, PrefLiveUpdateSet, False);
+
+	if (PrefLiveUpdateSet) {
+	}
+}
+
+int prefGetLiveUpdateInterval(void)
+{
+	if (PrefLiveUpdateSet)
+		return PrefLiveUpdateInterval;
+	return -PrefLiveUpdateInterval;
+}
