@@ -1,30 +1,36 @@
 /*
-  Xinvest Copyright (c)1995-96, 2000 
-  Mark Buser, All Rights Reserved.
-
-  Permission is hereby granted to copy
-  and freely distribute copies of this
-  program for non-commercial purposes
-  without fee, provided that this notice
-  appears in all copies.
-  
-  All redistributions must be in their
-  entirety as originally distributed.
-  Feel free to modify Xinvest, but
-  modified versions may not be distributed
-  without prior consent of the author.
-  
-  This software is provided 'as-is'
-  without any express or implied warranty.
-  In no event will the author be held
-  liable for any damages resulting from
-  the use of this software.
-
-  $Revision: 2.38 $ $Date: 2000/02/04 00:38:16 $
-*/
-
+ * Copyright (c) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, Mark Buser.
+ * Copyright (c) 2003, 2004, Danny Backx.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are
+ * met:
+ *
+ * Redistributions of source code must retain the above copyright notice,
+ * this list of conditions and the following disclaimer.
+ * Redistributions in binary form must reproduce the above copyright
+ * notice, this list of conditions and the following disclaimer in the
+ * documentation and/or other materials provided with the distribution.
+ * Neither the names the authors (see above), nor the names of other
+ * contributors may be used to endorse or promote products derived from
+ * this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * $Header: /pack/anoncvs/xinvest/src/rate.c,v 2.42 2004/04/24 07:58:40 danny Exp $
+ */
 /*  rate of return of a portfolio	*/
-
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -441,9 +447,7 @@ double rate_exch( int account, long date )
   return (exchg);
 }
 
-
-
-double rate_price( int account, long date )
+double rate_price(int account, long date, int allSplit)
 {
   int i;
   double price = 0.0;
@@ -486,11 +490,13 @@ double rate_price( int account, long date )
       if (type == T_CURR) /* latest exchange rate date */
         latestETrans = i;
 
-    /* We have to check all forward transactions to find any splits */
-    if ( nextDate > end && type == T_SPLIT )
-      /* Note inversion, price is effected opposite of shares by split */
-      split = (split * GetTransSplit(trans,i,TRANS_MINOR)) 
+    if (allSplit == TRUE) {
+      /* We have to check all forward transactions to find any splits */
+      if ( nextDate > end && type == T_SPLIT ) 
+        /* Note inversion, price is effected opposite of shares by split */
+        split = (split * GetTransSplit(trans,i,TRANS_MINOR)) 
 	       / GetTransSplit(trans,i,TRANS_MAJOR);
+    }
   }
 
   /* Pending price point? If not use last NAV within date range. */
@@ -517,35 +523,39 @@ double rate_price( int account, long date )
   /* Pending exchange quote? If not enabled or not available,   */
   /* use last exchange rate within date range.                  */
   if (prefGetUseCurr()) {
-    NAV *navp;
-    float val;
-    getAccount (account, ACCOUNT_CURRENCY, &navp);
-    if (latestETrans > 0)
-      exchg = GetTransCurrency (trans, latestETrans);
-    if (navp && isNavChanged(account, navp)) {
-      if ((latestETrans == 0 || (latestETrans > 0 &&
-	   navToLDate(navp) > GetTransLDate(trans, latestETrans))) &&
-	  navToLDate(navp) <= end ) {
-        getNav (navp, NAV_VALUE, &val);
-        exchg = (double)val;
-      }
-    }
+	  /* Check whether we have a currency account */
+	  int	ca;
+	  if ((ca = Account2CurrencyAccount(account))) {
+		  exchg = GetCurrencyForDate(ca, date);
+	  } else {
+	    NAV *navp;
+	    float val;
+	    getAccount (account, ACCOUNT_CURRENCY, &navp);
+	    if (latestETrans > 0)
+	      exchg = GetTransCurrency (trans, latestETrans);
+	    if (navp && isNavChanged(account, navp)) {
+	      if ((latestETrans == 0 || (latestETrans > 0 &&
+		   navToLDate(navp) > GetTransLDate(trans, latestETrans))) &&
+		  navToLDate(navp) <= end ) {
+		getNav (navp, NAV_VALUE, &val);
+		exchg = (double)val;
+	      }
+	    }
+	  }
   }
 
   return (price*exchg);
 }
 
-
 /* How many shares are in the account, taking into consideration dividend
 ** reinvestment (or not), load on sales, and splits. */
-double rate_shares( int account, long sdate, long edate )
+double rate_shares(int account, long sdate, long edate, int allSplit)
 {
   int i;
   double shares = 0.0;
 
   int numTrans;
   void *trans;
-  int type;
   int reinvest;
   long start, end;
 
@@ -565,30 +575,43 @@ double rate_shares( int account, long sdate, long edate )
     end = edate;
 
   for ( i=1; i <= numTrans; i++) {
+    long nextDate;
+    int type;
+	    
+    nextDate = GetTransLDate (trans, i);
+    type = GetTransType(trans,i);
 
-    if ( GetTransLDate(trans, i) < start ) 
+    if ( nextDate < start ) 
       continue;
     
-    if ( GetTransLDate(trans, i) > end ) 
-      break;
+    if ( nextDate > end ) {
+      if (allSplit == TRUE) {
+        /* We have to check all forward transactions to find any splits */
+        if ( nextDate > end && type == T_SPLIT ) 
+          shares = ( shares * GetTransSplit(trans,i,TRANS_MAJOR)) 
+	         / GetTransSplit(trans,i,TRANS_MINOR);
+      } else {
+        break;
+      }
+    } else {
 
-    /* add up all shares owned. */
-    type = GetTransType(trans,i);
-    reinvest = GetTransFlag ( trans, i, TRANS_REINVEST );
+      /* add up all shares owned. */
+      reinvest = GetTransFlag ( trans, i, TRANS_REINVEST );
 
-    /* shares are neg for T_SELL. divs only if reinvest on */
-    if ( type != T_DIST || reinvest )
-      shares += GetTransShares(trans, i);
+      /* shares are neg for T_SELL. divs only if reinvest on */
+      if ( type != T_DIST || reinvest )
+        shares += GetTransShares(trans, i);
 
-    /* possible load on sale */
-    if ( type == T_SELL )
-      shares -= GetTransLoad(trans,i) / GetTransNav(trans,i);
+      /* possible load on sale */
+      if ( type == T_SELL )
+        shares -= GetTransLoad(trans,i) / GetTransNav(trans,i);
+
+      if ( type == T_SPLIT )
+        shares = ( shares * GetTransSplit(trans,i,TRANS_MAJOR)) 
+  	         / GetTransSplit(trans,i,TRANS_MINOR);
+    }
 
-    if ( type == T_SPLIT )
-      shares = ( shares * GetTransSplit(trans,i,TRANS_MAJOR)) 
-	       / GetTransSplit(trans,i,TRANS_MINOR);
   }
-
   return ( shares );
 }
 
@@ -659,7 +682,8 @@ double rate_value ( int account, long sdate, long edate )
   else
     end = edate;
 
-  curtot = (rate_shares (account, start, end) * rate_price (account, end)) 
+  curtot = (rate_shares (account, start, end, TRUE)
+		  * rate_price (account, end, TRUE)) 
 	   - rate_esop_cost(account, start, end);
 
   return (curtot);
@@ -897,7 +921,7 @@ double rate_yield( int account, long sdate, long edate, YIELD *result )
       yield = GetTransShares(trans, i);
 
       /* / start shares */
-      shares = rate_shares (account, FIRST_DATE, GetTransLDate(trans, i-1));
+      shares = rate_shares(account, FIRST_DATE, GetTransLDate(trans, i-1), TRUE);
       if (shares > 0.0) {
 	exch = rate_exch (account, date);
 
@@ -916,7 +940,7 @@ double rate_yield( int account, long sdate, long edate, YIELD *result )
   }
 
   if (result) {
-    double years = (double)(end - start) / 365.0;
+    double years = (double)(end - start + 1.0) / 365.0;
     result->div = div;
     result->value = value;
 
