@@ -1,7 +1,8 @@
 "Xinvest.title: 				Xinvest 2.6",
 "Xinvest.fontList: -*-helvetica-bold-r-*-*-*-100-*-*-*-*-*-*=small,\
 -*-helvetica-bold-r-*-*-*-140-*-*-*-*-*-*=large",
-"*Butform.fractionBase:			7",
+"Xinvest.printFontList: -*-helvetica-bold-r-*-*-*-100-*-*-*-*-*-*=small,\
+-*-helvetica-bold-r-*-*-*-140-*-*-*-*-*-*=large",
 "*Quit.topPosition:			0",
 "*About.topPosition:			1",
 "*Edit.topPosition:			2",
@@ -9,6 +10,7 @@
 "*Graphbut.topPosition:			4",
 "*Portfolio.topPosition:			5",
 "*Calculate.topPosition:			6",
+"*Print.topPosition:			7",
 "*Quit.labelString:			Exit",
 "*About.labelString:			About Xinvest",
 "*Edit.labelString:			Transaction Editor",
@@ -426,6 +428,10 @@
 "*PrefPrecOpt.button_6.labelString:	6",
 "*PrefPref*ButForm.button_0.labelString:	Ok",
 "*PrefPref*ButForm.button_1.labelString:	Cancel",
+"*PrefLiveUpdate.labelString:		Live Update",
+"*liveUpdateToggle.labelString:		Save live update values",
+"*liveUpdateLabel.labelString:		Number of days between saves",
+"*PrefLiveUpdate.form_4.text_0.columns:	4",
 "*Help*topicframe.shadowType:		SHADOW_ETCHED_IN",
 "*Help*topiclist.visibleItemCount:	15",
 "*Help*textframe.shadowType:		SHADOW_ETCHED_IN",
@@ -449,10 +455,6 @@
 "*Report.dialogTitle:			Report",
 "*Report.allowShellResize:		True",
 "*ReportScroll.scrollBarDisplayPolicy:	static",
-"*ReportScroll.width:			670",
-"*ReportScroll.height:			240",
-"*ReportDrawing.width:			670",
-"*ReportDrawing.height:			240",
 "*Report.value.labelString:		Value",
 "*Report.price.labelString:		Price",
 "*Report.share.labelString:		Shares",
@@ -484,3 +486,7 @@
 "Xinvest.colorGrey:			Grey60",
 "Xinvest.colorWhite:			White",
 "Xinvest.colorBlack:			Black",
+"*colorPrintBackground:			white",
+"*colorPrintForeground:			black",
+"*colorPrintIllustrations:		blue",
+"*maxaccounts:				50",
