@@ -1,28 +1,35 @@
 /*
-  Xinvest Copyright (c)1995-98, 2000 
-  Mark Buser, All Rights Reserved.
-
-  Permission is hereby granted to copy
-  and freely distribute copies of this
-  program for non-commercial purposes
-  without fee, provided that this notice
-  appears in all copies.
-  
-  All redistributions must be in their
-  entirety as originally distributed.
-  Feel free to modify Xinvest, but
-  modified versions may not be distributed
-  without prior consent of the author.
-  
-  This software is provided 'as-is'
-  without any express or implied warranty.
-  In no event will the author be held
-  liable for any damages resulting from
-  the use of this software.
-
-  $Revision: 2.24 $ $Date: 2000/02/04 22:46:22 $
-*/
-
+ * Copyright (c) 1994, 1995, 1996, 1997, 1998, 2000, Mark Buser.
+ * Copyright (c) 2003, 2004, Danny Backx.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are
+ * met:
+ *
+ * Redistributions of source code must retain the above copyright notice,
+ * this list of conditions and the following disclaimer.
+ * Redistributions in binary form must reproduce the above copyright
+ * notice, this list of conditions and the following disclaimer in the
+ * documentation and/or other materials provided with the distribution.
+ * Neither the names the authors (see above), nor the names of other
+ * contributors may be used to endorse or promote products derived from
+ * this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * $Header: /pack/anoncvs/xinvest/src/trans.c,v 2.30 2004/04/24 07:58:40 danny Exp $
+ */
 #include <ctype.h>
 #include <math.h>
 #include <stdlib.h>
@@ -36,6 +43,7 @@
 #include "transP.h"
 #include "status.h"
 #include "util.h"
+#include "account.h"
 
 /* globals */
 static int lineno = 0;
@@ -149,9 +157,18 @@ double GetTransShares( void *trans, int num )
 /* 
 ** Return exchange rate of transction "num"
 */
-double GetTransCurrency( void *trans, int num )
+double GetTransCurrency(void *trans, int num)
 {
-  return ( ((TRANS *)trans)[num].t_currency );
+	int	ca = 0;
+	TRANS	*tp = &((TRANS *)trans)[num];
+
+#if 0
+	fprintf(stderr, "GetTransCurrency(%s)\n", GetAccountTitle(tp->t_account));
+#endif
+	if (tp->t_account && (ca = Account2CurrencyAccount(tp->t_account))) {
+		return GetCurrencyForDate(ca, tp->t_ldate);
+	}
+	return tp->t_currency;
 }
 
 /*
@@ -182,7 +199,7 @@ int GetTransSplit( void *trans, int num, int which )
 ** Read in a transaction from trans, reset to 1st line on new file.
 ** Return 0 on ok, -1 on error.
 */
-char *rd_trans( char *transline, int new )
+char *rd_trans(int account, char *transline, int new )
 {
     TRANS *tp;                  /* ptr to slots in trans[] array */
 
@@ -401,6 +418,9 @@ char *rd_trans( char *transline, int new )
 	   ecticker = XtNewString(token);
 	 }
        }
+#if 0
+       fprintf(stderr, "CTicker(%s,%s)\n", cticker, ecticker);
+#endif
        XtFree (newline);
        return (strlen(err_msg)?err_msg:0);
 
@@ -471,7 +491,25 @@ char *rd_trans( char *transline, int new )
     else
       tp -> t_flag &= ~TRANS_REINVEST;
     tp -> t_currency = curr;
-    
+    tp->t_account = account;
+#if 0
+    fprintf(stderr, "rd_trans t_account %d (%s)\n", account, GetAccountTitle(account));
+#endif 
     XtFree (newline);
     return ( NULL );
 }
+
+#include "account.h"
+
+long GetLastLDate(int acc)
+{
+	int	ntrans;
+	long	d;
+	TRANS	*trans;
+
+	getAccount(acc, ACCOUNT_TRANS, &trans);
+	getAccount(acc, ACCOUNT_NUM_TRANS, &ntrans);
+	d = GetTransLDate(trans, ntrans);
+
+	return d;
+}
