@@ -1,28 +1,35 @@
 /*
-  Xinvest Copyright (c)1995-98, 2000
-  Mark Buser, All Rights Reserved.
-
-  Permission is hereby granted to copy
-  and freely distribute copies of this
-  program for non-commercial purposes
-  without fee, provided that this notice
-  appears in all copies.
-  
-  All redistributions must be in their
-  entirety as originally distributed.
-  Feel free to modify Xinvest, but
-  modified versions may not be distributed
-  without prior consent of the author.
-  
-  This software is provided 'as-is'
-  without any express or implied warranty.
-  In no event will the author be held
-  liable for any damages resulting from
-  the use of this software.
-
-  $Revision: 2.11 $ $Date: 2000/02/04 00:37:46 $
-*/
-
+ * Copyright (c) 1994, 1995, 1996, 1997, 1998, 1999, 2000, Mark Buser.
+ * Copyright (c) 2003, 2004, Danny Backx.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are
+ * met:
+ *
+ * Redistributions of source code must retain the above copyright notice,
+ * this list of conditions and the following disclaimer.
+ * Redistributions in binary form must reproduce the above copyright
+ * notice, this list of conditions and the following disclaimer in the
+ * documentation and/or other materials provided with the distribution.
+ * Neither the names the authors (see above), nor the names of other
+ * contributors may be used to endorse or promote products derived from
+ * this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * $Header: /pack/anoncvs/xinvest/src/trans.h,v 2.15 2004/04/18 10:58:37 danny Exp $
+ */
 #define T_BUY 	      1  /* For access routines, types of transactions */
 #define T_SELL        2
 #define T_DIST        3
@@ -43,7 +50,7 @@
 int AreTrans();           /* Are there any transactions, returns how many */
 
 /* Parse transaction line, new file if int is non-zero */
-char *rd_trans( char *, int);
+char *rd_trans(int, char *, int);
 
 void *GetTrans();    /* Return pointer to current transactions, NULL on none */
 
@@ -70,4 +77,5 @@ int GetTransFlag (void *, int, int); /* Value of requested flag       */
 #define TRANS_MAJOR 1
 #define TRANS_MINOR 2
 int GetTransSplit (void *, int, int);/* Value of requested split.     */
+long GetLastLDate(int acc);
 
