@@ -1,29 +1,40 @@
 /*
-  Xinvest Copyright (c)1995-98 Mark Buser,
-  All Rights Reserved.
-
-  Permission is hereby granted to copy
-  and freely distribute copies of this
-  program for non-commercial purposes
-  without fee, provided that this notice
-  appears in all copies.
-  
-  All redistributions must be in their
-  entirety as originally distributed.
-  Feel free to modify Xinvest, but
-  modified versions may not be distributed
-  without prior consent of the author.
-  
-  This software is provided 'as-is'
-  without any express or implied warranty.
-  In no event will the author be held
-  liable for any damages resulting from
-  the use of this software.
-
-  $Revision: 1.14 $ $Date: 1998/05/02 15:58:38 $
-*/
+ * Copyright (c) 1997, 2000, Mark Buser.
+ * Copyright (c) 2003, Danny Backx.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are
+ * met:
+ *
+ * Redistributions of source code must retain the above copyright notice,
+ * this list of conditions and the following disclaimer.
+ * Redistributions in binary form must reproduce the above copyright
+ * notice, this list of conditions and the following disclaimer in the
+ * documentation and/or other materials provided with the distribution.
+ * Neither the names the authors (see above), nor the names of other
+ * contributors may be used to endorse or promote products derived from
+ * this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * $Header: /pack/anoncvs/xinvest/src/util.c,v 1.11 2004/04/30 16:00:18 danny Exp $
+ */
 #include <errno.h>
 #include <limits.h>
+#ifdef	HAVE_MONETARY_H
+#include <monetary.h>
+#endif
 #include <stdio.h>
 #include <string.h>
 #include <time.h>
@@ -31,16 +42,43 @@
 
 #include <Xm/Xm.h>
 
+#ifndef XQUOTE   /* Xinvest specific headers */
 #include "account.h"
 #include "calendar.h"
-#include "file.h"
-#ifdef STRFMON
-#include "monetary.h"
-#endif
+#include "fileXi.h"
 #include "pref.h"
+#endif
+
+#ifdef	XQUOTE
+#include "xquote.h"
+#else
+#include "xinvest.h"
+#endif
+
 #include "status.h"
 #include "util.h"
 
+/*
+** Program Specific Support
+*/
+
+#ifdef XQUOTE
+void syntax(int argc, char **argv)
+{
+  if (argc != 1)
+    fprintf (stderr, "%s: unknown option \"%s\"\n", argv[0], argv[1]);
+  fprintf (stderr, "usage: %s [options ...]\n", argv[0]);
+  fprintf (stderr, 
+           "where options include:\n\n"
+           "-help               to print this message,\n"
+           "-install            to install a private colormap,\n"
+           "-tape               tape display with no menu bar,\n"
+           "-restore filename   to restore a saved state (a previous session),\n"
+           "or any valid X toolkit option.\n\n");
+}
+
+#else /* Xinvest */
+
 void syntax(int argc, char **argv)
 {
   if (argc != 1)
@@ -50,32 +88,10 @@ void syntax(int argc, char **argv)
   fprintf( stderr, "       -f filename(s)  to load transaction file(s),\n");
   fprintf( stderr, "       -help           to print this message,\n");
   fprintf( stderr, "       -install        to install a private colormap,\n");
+  fprintf( stderr, "       -restore        to restore a saved state (a previous session),\n");
   fprintf( stderr, "       or any valid X toolkit option.\n\n");
 }
 
-
-/* All I want is the lousy max path length, why is this a good thing? */
-#ifdef PATH_MAX
-static int pathmax = PATH_MAX;
-#else
-static int pathmax = 0;
-#endif
-
-#define PATH_MAX_GUESS 1024
-
-int path_length ()
-{
-   if (pathmax == 0) {
-     if ((pathmax = pathconf ("/", _PC_PATH_MAX)) < 0) 
-       pathmax = PATH_MAX_GUESS;
-     else
-       pathmax++;
-   }
-
-   return (pathmax);
-}
-
-
 /* Look at an argc/argv pair for the -f filename[s] option.  Return argc/argv
 ** with this option removed.
 */
@@ -100,8 +116,7 @@ void removeFileOption ( int *argc, char **argv )
 
     if ( start > 0 ) {          /* Found -f option. */
 
-      if ( end > 0 ) {          /* Option(s) after it, move up to discard -f. */
-        int newargc = *argc - (end - start);
+      if ( end > 0 ) {          /* Option(s) after it, move up to discard -f. */        int newargc = *argc - (end - start);
 
         for ( j = end; j < *argc; j++)
           argv[start++] = argv[j];
@@ -148,7 +163,7 @@ void processFileOption ( int argc, char **argv)
       argv++; argc--;                      /* skip -f */
 
     } else {                               /* -f option, as expected */
-      argv++; argc--; 
+      argv++; argc--;
 
       while ( argc ) {                     /* any number of filenames */
         if (strncmp ( "-", *argv, strlen("-")) == 0)
@@ -168,6 +183,7 @@ void processFileOption ( int argc, char **argv)
     displayAccount( account );
 }
 
+
 /* Format date from long format into preference format */
 char *utilDateOut (long dateIn)
 {
@@ -179,14 +195,14 @@ char *utilDateOut (long dateIn)
 
   switch (prefGetDate()) {
     case PREF_DATE_LOC:
-	 strftime (dateStr, 20, "%x", &dateTm);
+         strftime (dateStr, 20, "%x", &dateTm);
          break;
     case PREF_DATE_DMY:
-	 strftime (dateStr, 20, "%d/%m/%Y", &dateTm);
+         strftime (dateStr, 20, "%d/%m/%Y", &dateTm);
          break;
     default:
     case PREF_DATE_MDY:
-	 strftime (dateStr, 20, "%m/%d/%Y", &dateTm);
+         strftime (dateStr, 20, "%m/%d/%Y", &dateTm);
          break;
   }
   return (dateStr);
@@ -208,21 +224,22 @@ char *utilCurrOut (double currIn, int currency)
                        if (currency==UTILCURR)
                          strfmon (strval, 40, "%.2n", currIn);
                        else {
-			 char format[10];
-			 sprintf (format, "%%!.%dn", precision);
+                         char format[10];
+                         sprintf (format, "%%!.%dn", precision);
                          strfmon (strval, 40, format, currIn);
-		       }
+                       }
                        return (strval);
-#endif         
+#endif
 
-    case PREF_CURR_DC: 
-		       groupc = '.';
+    case PREF_CURR_DC:
+                       groupc = '.';
                        decc = ',';
                        break;
 
+
     case PREF_CURR_CD:
     default:
-		       groupc = ',';
+                       groupc = ',';
                        decc = '.';
                        break;
   }
@@ -233,14 +250,14 @@ char *utilCurrOut (double currIn, int currency)
   leadd = strspn (tmp, "0123456789");
   to = strval;
   from = tmp;
-    
+
   for (i=0; i < strlen(tmp) && i < 39; i++) {
     if (i==0 && currency==UTILCURR) {
       strcpy (to, prefGetCurrChar());
       to += strlen (prefGetCurrChar());
     }
-    if (leadd) 
-      if (leadd--%3 == 0 && i != 0) 
+    if (leadd)
+      if (leadd--%3 == 0 && i != 0)
         *to++ = groupc;
     if (*from == '.' || *from == '.') {
       *to++ = decc;
@@ -249,14 +266,37 @@ char *utilCurrOut (double currIn, int currency)
       *to++ = *from++;
   }
   *to = '\0';
-                       
+
   return (strval);
 }
 
+#endif /* Xinvest */
+
 /*
 ** Host Specific Support
 */
 
+/* All I want is the lousy max path length, why is this a good thing? */
+#ifdef PATH_MAX
+static int pathmax = PATH_MAX;
+#else
+static int pathmax = 0;
+#endif
+
+#define PATH_MAX_GUESS 1024
+
+int path_length ()
+{
+   if (pathmax == 0) {
+     if ((pathmax = pathconf ("/", _PC_PATH_MAX)) < 0) 
+       pathmax = PATH_MAX_GUESS;
+     else
+       pathmax++;
+   }
+
+   return (pathmax);
+}
+
 /* Supplied by Cube-X */
 #ifdef NeXT
 
@@ -330,13 +370,14 @@ strcasecmp(char *s1, char *s2)
 
 #if defined (NEED_STRPTIME)
 
-/* Weird, FreeBSD (< 2.2?) and IRIX 5.x don't have this, but do have strftime. 
+/* Weird, FreeBSD (< 2.2?) and IRIX 5 don't have this, but do have strftime. 
 ** This is a crippled implementation, only supporting the formats
-** actually used in the rest of Xinvest.  Note that the return _value_
+** actually used in the rest of Xquote.  Note that the return _value_
 ** does not conform to the man page either.
 */
 char *strptime ( char* buf, char *fmt, struct tm *tm )
 {
+  char err_msg[128];
 
   if ( strcmp( fmt, "%m/%d/%y" ) == 0 ) 
     sscanf ( buf, "%d/%d/%d", &tm->tm_mon, &tm->tm_mday, &tm->tm_year );
@@ -349,9 +390,9 @@ char *strptime ( char* buf, char *fmt, struct tm *tm )
     sscanf ( buf, "%d %d %d", &tm->tm_mon, &tm->tm_mday, &tm->tm_year );
 
   else {
-    sprintf ( errmsg, "Xinvest internal error: strptime.  Unsupported\n"
+    sprintf ( err_msg, "internal error: strptime.  Unsupported\n"
                       "format: \"%s\".  See Reporting Bugs.", fmt);
-    write_status( errmsg, ERR);
+    write_status( err_msg, ERR);
   }
 
   tm->tm_mon--;  /* Months are from 0-11, mdays from 1-31, years since 1900 */
@@ -359,3 +400,30 @@ char *strptime ( char* buf, char *fmt, struct tm *tm )
   return NULL;  /* completely bogus return value, but I don't use it */
 }
 #endif
+
+
+#ifdef NEED_STRERROR
+/* Supplied by Benoit Pirenne <bpirenne@eso.org> */
+
+#include <errno.h>
+
+extern char *sys_errlist[];
+extern int  sys_nerr;
+ 
+char *strerror (errnum)
+/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+.PURPOSE Retrieve the corresponding msg for an given errno
+.REMARK  Only useful on SunOS. Service provided on Solaris!
+----------------------------------------------------------------------------*/
+int errnum;                   /* IN: errno value */
+{
+    char *p;
+ 
+    if ( errnum <= 0 ) p = "";
+    else
+      if ( errnum <= sys_nerr)
+        p = sys_errlist[errnum];
+ 
+    return p;
+}
+#endif
