@@ -1,30 +1,36 @@
 /*
-  Xinvest/Xquote Copyright 1995-97 Mark Buser,
-  All Rights Reserved.
-
-  Permission is hereby granted to copy
-  and freely distribute copies of this
-  program for non-commercial purposes
-  without fee, provided that this notice
-  appears in all copies.
-  
-  All redistributions must be in their
-  entirety as originally distributed.
-  Feel free to modify Xinvest/Xquote, but
-  modified versions may not be distributed
-  without prior consent of the author.
-  
-  This software is provided 'as-is'
-  without any express or implied warranty.
-  In no event will the author be held
-  liable for any damages resulting from
-  the use of this software.
-
-  $Revision: 2.10 $ $Date: 1997/12/07 21:42:25 $
-*/
-
+ * Copyright (c) 1997, 1999, 2000, Mark Buser.
+ * Copyright (c) 2001, 2003, Danny Backx.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are
+ * met:
+ *
+ * Redistributions of source code must retain the above copyright notice,
+ * this list of conditions and the following disclaimer.
+ * Redistributions in binary form must reproduce the above copyright
+ * notice, this list of conditions and the following disclaimer in the
+ * documentation and/or other materials provided with the distribution.
+ * Neither the names the authors (see above), nor the names of other
+ * contributors may be used to endorse or promote products derived from
+ * this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+ * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * $Header: /pack/anoncvs/xinvest/src/xutil.c,v 1.11 2003/11/19 20:00:09 danny Exp $
+ */
 #include <stdio.h>
-
 #include <Xm/XmAll.h>
 
 #ifndef XQUOTE
@@ -295,6 +301,9 @@ void setBusyCursor(Widget w, int state)
   Display *dsp;
   Window win;
  
+  /* FIX ME */
+  return;
+
   if ( XtIsRealized (top) ) {
     dsp = XtDisplay (top);
     win = XtWindow (top);
@@ -316,25 +325,26 @@ void setBusyCursor(Widget w, int state)
 int isCDE()
 {
   Display *dpy = XOpenDisplay (NULL);
-  Window root = DefaultRootWindow (dpy);
+  Window root;
   Atom atom, type;
   int  format;
   unsigned long nitems, remaining;
   unsigned char *prop;
 
   if (dpy) {
-    atom = XInternAtom( dpy, "_DT_SM_WINDOW_INFO", True );
-    if (atom != None) {
-      if (XGetWindowProperty (dpy, root, atom,
+	root = DefaultRootWindow (dpy);
+	atom = XInternAtom( dpy, "_DT_SM_WINDOW_INFO", True );
+	if (atom != None) {
+		if (XGetWindowProperty (dpy, root, atom,
                               0, (65536 / sizeof (long)),
                               False,
                               XA_STRING,
                               &type, &format,
                               &nitems, &remaining,
                               &prop)
-          == Success)
-        return True;
-    }
+			== Success)
+			return True;
+	}
 
     atom = XInternAtom( dpy, "_VUE_SM_WINDOW_INFO", True );
     if (atom != None) {
@@ -402,13 +412,15 @@ void PostIt ( Widget w, XtPointer client_data, XEvent *event)
 {
   Widget popup = (Widget) client_data;
   XButtonPressedEvent *bevent = (XButtonPressedEvent *)event;
+  int data;
+
+  /* Some may wish to pass user data from parent to the menu callback */
+  XtVaGetValues (w, XmNuserData, &data, NULL);
 
-  /* Should only be button 3, but if we're in a CDE front panel, we'll
-  ** never see it; how to allow button 2 as well? */
-  if (bevent->button != 3)
-    return;
   XmMenuPosition (popup, bevent);
   XtManageChild (popup);
+
+  XtVaSetValues (popup, XmNuserData, data, NULL);
 }
 
 /*
